# coding: utf-8
"""Tests for the Python API for composing notebook elements"""

from nbformat.validator import isvalid, validate, ValidationError
from ..nbbase import (
    NotebookNode, nbformat,
    new_code_cell, new_markdown_cell, new_notebook,
    new_output, new_raw_cell,
)

def test_empty_notebook():
    nb = new_notebook()
    assert nb.cells == []
    assert nb.metadata == NotebookNode()
    assert nb.nbformat == nbformat

def test_empty_markdown_cell():
    cell = new_markdown_cell()
    assert cell.cell_type == 'markdown'
    assert cell.source == ''

def test_markdown_cell():
    cell = new_markdown_cell(u'* Søme markdown')
    assert cell.source == u'* Søme markdown'

def test_empty_raw_cell():
    cell = new_raw_cell()
    assert cell.cell_type == u'raw'
    assert cell.source == ''

def test_raw_cell():
    cell = new_raw_cell('hi')
    assert cell.source == u'hi'

def test_empty_code_cell():
    cell = new_code_cell('hi')
    assert cell.cell_type == 'code'
    assert cell.source == u'hi'

def test_empty_display_data():
    output = new_output('display_data')
    assert output.output_type == 'display_data'

def test_empty_stream():
    output = new_output('stream')
    assert output.output_type == 'stream'
    assert output.name == 'stdout'
    assert output.text == ''

def test_empty_execute_result():
    output = new_output('execute_result', execution_count=1)
    assert output.output_type == 'execute_result'

mimebundle = {
    'text/plain': "some text",
    "application/json": {
        "key": "value"
    },
    "image/svg+xml": 'ABCDEF',
    "application/octet-stream": 'ABC-123',
    "application/vnd.foo+bar": "Some other stuff",
}

def test_display_data():
    output = new_output('display_data', mimebundle)
    for key, expected in mimebundle.items():
        assert output.data[key] == expected

def test_execute_result():
    output = new_output('execute_result', mimebundle, execution_count=10)
    assert output.execution_count == 10
    for key, expected in mimebundle.items():
        assert output.data[key] == expected

def test_error():
    o = new_output(output_type=u'error', ename=u'NameError',
        evalue=u'Name not found', traceback=[u'frame 0', u'frame 1', u'frame 2']
    )
    assert o.output_type == u'error'
    assert o.ename == u'NameError'
    assert o.evalue == u'Name not found'
    assert o.traceback == [u'frame 0', u'frame 1', u'frame 2']

def test_code_cell_with_outputs():
    cell = new_code_cell(execution_count=10, outputs=[
        new_output('display_data', mimebundle),
        new_output('stream', text='hello'),
        new_output('execute_result', mimebundle, execution_count=10),
    ])
    assert cell.execution_count == 10
    assert len(cell.outputs) == 3
    er = cell.outputs[-1]
    assert er.execution_count == 10
    assert er['output_type'] == 'execute_result'

def test_stream():
    output = new_output('stream', name='stderr', text='hello there')
    assert output.name == 'stderr'
    assert output.text == 'hello there'



"""The main module for the v1 notebook format."""

#-----------------------------------------------------------------------------
#  Copyright (C) 2008-2011  The IPython Development Team
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

from .nbbase import (
    NotebookNode,
    new_code_cell, new_text_cell, new_notebook
)

from .nbjson import reads as reads_json, writes as writes_json
from .nbjson import reads as read_json, writes as write_json
from .nbjson import to_notebook as to_notebook_json

from .convert import upgrade



from unittest import TestCase

from ..nbbase import (
    NotebookNode,
    new_code_cell, new_text_cell, new_notebook
)

class TestCell(TestCase):

    def test_empty_code_cell(self):
        cc = new_code_cell()
        self.assertEqual(cc.cell_type,'code')
        self.assertEqual('code' not in cc, True)
        self.assertEqual('prompt_number' not in cc, True)

    def test_code_cell(self):
        cc = new_code_cell(code='a=10', prompt_number=0)
        self.assertEqual(cc.code, u'a=10')
        self.assertEqual(cc.prompt_number, 0)

    def test_empty_text_cell(self):
        tc = new_text_cell()
        self.assertEqual(tc.cell_type, 'text')
        self.assertEqual('text' not in tc, True)

    def test_text_cell(self):
        tc = new_text_cell('hi')
        self.assertEqual(tc.text, u'hi')


class TestNotebook(TestCase):

    def test_empty_notebook(self):
        nb = new_notebook()
        self.assertEqual(nb.cells, [])

    def test_notebooke(self):
        cells = [new_code_cell(),new_text_cell()]
        nb = new_notebook(cells=cells)
        self.assertEqual(nb.cells,cells)

